"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const jsymbol_1 = require("jsymbol");
const ast = require("./astsql");
const queryVisitor_1 = require("./queryVisitor");
var SymbolState;
(function (SymbolState) {
    SymbolState[SymbolState["Resolved"] = 0] = "Resolved";
    SymbolState[SymbolState["Unresolved"] = 1] = "Unresolved";
})(SymbolState = exports.SymbolState || (exports.SymbolState = {}));
class SqlAstSymbol {
    constructor(identifier, type) {
        this.identifier = identifier;
        this.type = type;
        this.state = SymbolState.Unresolved;
    }
}
exports.SqlAstSymbol = SqlAstSymbol;
class ValidationContext {
    constructor(currentNode) {
        this.currentNode = currentNode;
        this.symbolTable = new jsymbol_1.SymbolTable();
    }
}
exports.ValidationContext = ValidationContext;
class QueryValidator extends queryVisitor_1.QueryVisitor {
    constructor(dialect, query, contextBuilder = ValidationContext) {
        super(dialect, query, contextBuilder);
        this.visitSelectStatement = (context, node) => {
            context.symbolTable.enterScope();
            this.visitGenericNode(context, node.query);
            context.symbolTable.exitScope();
            return context;
        };
        this.visitQueryExpression = (context, node) => {
            if (node.from) {
                this.visitNode(context, node.from);
            }
            if (node.elements && node.elements.length) {
                for (let se of node.elements) {
                    this.visitNode(context, se);
                }
            }
            if (node.orderBy) {
                this.visitNode(context, node.orderBy);
            }
            if (node.limit) {
                this.visitNode(context, node.limit);
            }
            return context;
        };
        this.visitTableSpec = (context, node) => {
            let symbol = new SqlAstSymbol(node.tableName.toString(), ast.SqlSymbolType.Table);
            context.symbolTable.add(symbol);
            symbol.state = SymbolState.Resolved;
            node.tableName = symbol;
            if (node.partitions) {
                for (let p of node.partitions) {
                    let partitionField = new SqlAstSymbol(p.toString(), ast.SqlSymbolType.Field);
                    partitionField.parent = symbol;
                    partitionField.state = SymbolState.Resolved;
                    context.symbolTable.add(partitionField);
                }
            }
            return context;
        };
        this.visitAliasedTerm = (context, node) => {
            return context;
        };
    }
    validate() {
        let context = this.visit();
        return context.symbolTable;
    }
}
exports.QueryValidator = QueryValidator;
class MySQLQueryValidator extends QueryValidator {
    constructor(query) {
        super("MySQL", query, ValidationContext);
    }
}
exports.MySQLQueryValidator = MySQLQueryValidator;
//# sourceMappingURL=queryValidator.js.map